@@ -5,8 +5,8 @@
 ;; Author: Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2021
-;; Version: 2.3
-;; Package-Requires: ((emacs "28.1") (compat "30"))
+;; Version: 2.7
+;; Package-Requires: ((emacs "29.1") (compat "30"))
 ;; URL: https://github.com/minad/corfu
 ;; Keywords: abbrev, convenience, matching, completion, text
 
@@ -72,7 +72,7 @@ The value should lie between 0 and corfu-count/2."
   "Enable cycling for `corfu-next' and `corfu-previous'."
   :type 'boolean)
 
-(defcustom corfu-on-exact-match 'insert
+(defcustom corfu-on-exact-match nil
   "Configure how a single exact match should be handled.
 - nil: No special handling, continue completion.
 - insert: Insert candidate, quit and call the `:exit-function'.
@@ -84,7 +84,7 @@ The value should lie between 0 and corfu-count/2."
   '(ignore universal-argument universal-argument-more digit-argument
     "\\`corfu-" "\\`scroll-other-window")
   "Continue Corfu completion after executing these commands.
-The list can container either command symbols or regular expressions."
+The list can contain either command symbols or regular expressions."
   :type '(repeat (choice regexp symbol)))
 
 (defcustom corfu-preview-current 'insert
@@ -140,6 +140,10 @@ separator: Only stay alive if there is no match and
   "Width of the bar in units of the character width."
   :type 'float)
 
+(defcustom corfu-border-width 1
+  "Width of the border in pixels, only applies to GUI Emacs."
+  :type 'natnum)
+
 (defcustom corfu-margin-formatters nil
   "Registry for margin formatter functions.
 Each function of the list is called with the completion metadata as
@@ -167,26 +171,33 @@ This function is used even if a completion table specifies its
 own sort function."
   :type '(choice (const nil) function))
 
+(defcustom corfu-auto-trigger ""
+  "Characters which trigger auto completion.
+If a trigger character is detected `corfu-auto-prefix' is ignored."
+  :type 'string)
+
 (defcustom corfu-auto-prefix 3
   "Minimum length of prefix for auto completion.
-The completion backend can override this with
-:company-prefix-length.  It is *not recommended* to use a small
-prefix length (below 2), since this will create high load for
-Emacs.  See also `corfu-auto-delay'."
+The completion backend can override this with :company-prefix-length.
+It is not recommended to use a small prefix length (below 2), since this
+will create high load for Emacs.  See also `corfu-auto-delay' and
+`corfu-auto-trigger'."
   :type 'natnum)
 
 (defcustom corfu-auto-delay 0.2
   "Delay for auto completion.
-It is *not recommended* to use a short delay or even 0, since
-this will create high load for Emacs, in particular if executing
-the completion backend is costly."
+It is not recommended to use a short delay or even 0, since this will
+create high load for Emacs, in particular if executing the completion
+backend is costly.  Instead of reducing the delay too much, try
+`corfu-auto-trigger' to trigger immediate completion after certain
+characters."
   :type 'float)
 
 (defcustom corfu-auto-commands
   '("self-insert-command\\'" "delete-backward-char\\'" "\\`backward-delete-char"
     c-electric-colon c-electric-lt-gt c-electric-slash c-scope-operator)
   "Commands which initiate auto completion.
-The list can container either command symbols or regular expressions."
+The list can contain either command symbols or regular expressions."
   :type '(repeat (choice regexp symbol)))
 
 (defcustom corfu-auto nil
@@ -194,7 +205,7 @@ The list can container either command symbols or regular expressions."
 Auto completion is disabled by default for safety and unobtrusiveness.
 Note that auto completion is particularly dangerous in untrusted files
 since some completion functions may perform arbitrary code execution,
-notably the Emacs built-in `elisp-completion-at-point' .  See also the
+notably the Emacs built-in `elisp-completion-at-point'.  See also the
 settings `corfu-auto-delay', `corfu-auto-prefix' and
 `corfu-auto-commands'."
   :type 'boolean)
@@ -207,6 +218,7 @@ settings `corfu-auto-delay', `corfu-auto-prefix' and
 (defface corfu-default
   '((((class color) (min-colors 88) (background dark)) :background "#191a1b")
     (((class color) (min-colors 88) (background light)) :background "#f0f0f0")
+    (((background dark)) :background "gray" :foreground "black")
     (t :background "gray"))
   "Default face, foreground and background colors used for the popup.")
 
@@ -215,13 +227,13 @@ settings `corfu-auto-delay', `corfu-auto-prefix' and
      :background "#00415e" :foreground "white" :extend t)
     (((class color) (min-colors 88) (background light))
      :background "#c0efff" :foreground "black" :extend t)
-    (t :background "blue" :foreground "white" :extend t))
+    (t :background "magenta" :foreground "white" :extend t))
   "Face used to highlight the currently selected candidate.")
 
 (defface corfu-bar
   '((((class color) (min-colors 88) (background dark)) :background "#a8a8a8")
     (((class color) (min-colors 88) (background light)) :background "#505050")
-    (t :background "gray"))
+    (t :background "black"))
   "The background color is used for the scrollbar indicator.")
 
 (defface corfu-border
@@ -331,8 +343,6 @@ settings `corfu-auto-delay', `corfu-auto-prefix' and
     (min-height . t)
     (border-width . 0)
     (outer-border-width . 0)
-    (internal-border-width . 1)
-    (child-frame-border-width . 1)
     (vertical-scroll-bars . nil)
     (horizontal-scroll-bars . nil)
     (menu-bar-lines . 0)
@@ -342,11 +352,13 @@ settings `corfu-auto-delay', `corfu-auto-prefix' and
     (no-other-frame . t)
     (unsplittable . t)
     (undecorated . t)
+    (fullscreen . nil)
     (cursor-type . nil)
     (no-special-glyphs . t)
     (desktop-dont-save . t)
     (inhibit-double-buffering . t)) ;; Avoid display artifacts on X/Gtk builds
-  "Default child frame parameters.")
+  "Default child frame parameters.
+It is recommended to avoid changing these parameters.")
 
 (defvar corfu--buffer-parameters
   '((mode-line-format . nil)
@@ -369,7 +381,8 @@ settings `corfu-auto-delay', `corfu-auto-prefix' and
     (indicate-buffer-boundaries . nil)
     (buffer-read-only . t)
     (pixel-scroll-precision-mode . nil))
-  "Default child frame buffer parameters.")
+  "Default child frame buffer parameters.
+It is recommended to avoid changing these parameters.")
 
 (defvar corfu--mouse-ignore-map
   (let ((map (define-keymap "<touchscreen-begin>" #'ignore)))
@@ -400,18 +413,28 @@ is a prefix length override, which is t for manual completion."
           (let ((len (or prefix
                          (plist-get plist :company-prefix-length)
                          (- (point) beg))))
-            (or (eq len t) (>= len corfu-auto-prefix)))
+            (or (eq len t) (>= len corfu-auto-prefix)
+                (seq-contains-p corfu-auto-trigger last-command-event)))
           ;; For non-exclusive Capfs, check for valid completion.
           (or (not (eq 'no (plist-get plist :exclusive)))
-              (let* ((str (buffer-substring-no-properties beg end))
-                     (pt (- (point) beg))
-                     (pred (plist-get plist :predicate))
-                     (md (completion-metadata (substring str 0 pt) table pred)))
-                ;; We use `completion-try-completion' to check if there are
-                ;; completions. The upstream `completion--capf-wrapper' uses
-                ;; `try-completion' which is incorrect since it only checks for
-                ;; prefix completions.
-                (completion-try-completion str table pred pt md)))
+              (let ((str (buffer-substring-no-properties beg end))
+                    (pt (- (point) beg))
+                    (pred (plist-get plist :predicate)))
+                ;; NOTE:
+                ;; * We use `completion-try-completion' to check if there are
+                ;;   completions. The upstream `completion--capf-wrapper' uses
+                ;;   `try-completion' which is incorrect since it only checks
+                ;;   for prefix completions.
+                ;; * `eglot--dumb-tryc' always returns nil which is
+                ;;   exceptional. Even if `eglot-completion-at-point' is marked
+                ;;   non-exclusive via `cape-capf-nonexclusive', the
+                ;;   `eglot--dumb-flex' completion style must additionally be
+                ;;   overridden to not defeat the check here.
+                ;; * An alternative would be to use `completion-all-completions'
+                ;;   to explicitly check for the presence of candidates. This
+                ;;   can be made efficient with a side-effecting predicate which
+                ;;   throws on first successful match.
+                (corfu--try-completion str table pred pt)))
           (cons fun res)))))
 
 (defun corfu--make-buffer (name)
@@ -461,21 +484,32 @@ FRAME is the existing frame."
          (x-gtk-resize-child-frames corfu--gtk-resize-child-frames)
          (before-make-frame-hook)
          (after-make-frame-functions)
-         (parent (window-frame)))
+         (parent (window-frame))
+         (graphic (display-graphic-p parent))
+         (params `((background-color
+                    . ,(face-attribute 'corfu-default :background nil 'default))
+                   (font . ,(frame-parameter parent 'font))
+                   (right-fringe . ,right-fringe-width)
+                   (left-fringe . ,left-fringe-width)
+                   (internal-border-width . ,corfu-border-width)
+                   (child-frame-border-width . ,corfu-border-width)
+                   ,@corfu--frame-parameters)))
     (unless (and (frame-live-p frame)
                  (eq (frame-parent frame)
-                     (and (not (bound-and-true-p exwm--connection)) parent))
+                     (and (not (and graphic (bound-and-true-p exwm--connection)))
+                          parent))
+                 ;; Handle mixed tty/graphical sessions
+                 (eq graphic (display-graphic-p frame))
                  ;; If there is more than one window, `frame-root-window' may
                  ;; return nil.  Recreate the frame in this case.
                  (window-live-p (frame-root-window frame)))
       (when frame (delete-frame frame))
       (setq frame (make-frame
-                   `((parent-frame . ,parent)
+                   `((name . ,(if graphic "EmacsCorfuGUI" "EmacsCorfuTTY"))
+                     (parent-frame . ,parent)
                      (minibuffer . ,(minibuffer-window parent))
                      (width . 0) (height . 0) (visibility . nil)
-                     (right-fringe . ,right-fringe-width)
-                     (left-fringe . ,left-fringe-width)
-                     ,@corfu--frame-parameters))))
+                     ,@params))))
     ;; XXX HACK Setting the same frame-parameter/face-background is not a nop.
     ;; Check before applying the setting. Without the check, the frame flickers
     ;; on Mac. We have to apply the face background before adjusting the frame
@@ -485,8 +519,7 @@ FRAME is the existing frame."
         (set-face-background 'internal-border new frame))
       ;; XXX The Emacs Mac Port does not support `internal-border', we also have
       ;; to set `child-frame-border'.
-      (unless (or (not (facep 'child-frame-border))
-                  (equal (face-attribute 'child-frame-border :background frame 'default) new))
+      (unless (equal (face-attribute 'child-frame-border :background frame 'default) new)
         (set-face-background 'child-frame-border new frame)))
     ;; Reset frame parameters if they changed.  For example `tool-bar-mode'
     ;; overrides the parameter `tool-bar-lines' for every frame, including child
@@ -494,13 +527,7 @@ FRAME is the existing frame."
     ;; lovely surprises.
     (let* ((win (frame-root-window frame))
            (is (frame-parameters frame))
-           (should `((background-color
-                      . ,(face-attribute 'corfu-default :background nil 'default))
-                     (font . ,(frame-parameter parent 'font))
-                     (right-fringe . ,right-fringe-width)
-                     (left-fringe . ,left-fringe-width)
-                     ,@corfu--frame-parameters))
-           (diff (cl-loop for p in should for (k . v) = p
+           (diff (cl-loop for p in params for (k . v) = p
                           unless (equal (alist-get k is) v) collect p)))
       (when diff (modify-frame-parameters frame diff))
       ;; XXX HACK: `set-window-buffer' must be called to force fringe update.
@@ -512,14 +539,19 @@ FRAME is the existing frame."
       ;; Mark window as dedicated to prevent frame reuse (gh:minad/corfu#60)
       (set-window-dedicated-p win t))
     (redirect-frame-focus frame parent)
-    (set-frame-size frame width height t)
     (pcase-let ((`(,px . ,py) (frame-position frame)))
-      (unless (and (= x px) (= y py))
-        (set-frame-position frame x y))))
+      (cond
+       ((and (= x px) (= y py)) (set-frame-size frame width height t))
+       ;; New Emacs 31 function for faster resizing/movement in one go.
+       ((fboundp 'set-frame-size-and-position-pixelwise)
+        (set-frame-size-and-position-pixelwise frame width height x y))
+       (t (set-frame-size frame width height t)
+          (set-frame-position frame x y)))))
   (make-frame-visible frame)
   ;; Unparent child frame if EXWM is used, otherwise EXWM buffers are drawn on
   ;; top of the Corfu child frame.
-  (when (and (bound-and-true-p exwm--connection) (frame-parent frame))
+  (when (and (bound-and-true-p exwm--connection)
+             (display-graphic-p frame) (frame-parent frame))
     (redisplay t)
     (set-frame-parameter frame 'parent-frame nil))
   frame)
@@ -535,10 +567,14 @@ FRAME is the existing frame."
 
 (defun corfu--hide-frame (frame)
   "Hide child FRAME."
-  (when (and (frame-live-p frame) (frame-visible-p frame)
-             (not (frame-parameter frame 'corfu--hide-timer)))
-    (set-frame-parameter frame 'corfu--hide-timer
-                         (run-at-time 0 nil #'corfu--hide-frame-deferred frame))))
+  (when (and (frame-live-p frame) (frame-visible-p frame))
+    (cond
+     ((not (display-graphic-p frame))
+      (corfu--hide-frame-deferred frame))
+     ((not (frame-parameter frame 'corfu--hide-timer))
+      (set-frame-parameter
+       frame 'corfu--hide-timer
+       (run-at-time 0 nil #'corfu--hide-frame-deferred frame))))))
 
 (defun corfu--move-to-front (elem list)
   "Move ELEM to front of LIST."
@@ -578,6 +614,11 @@ FRAME is the existing frame."
                     (and cands (nconc cands base)))))
         (cons (apply #'completion-all-completions args) completion-lazy-hilit-fn)))))
 
+(defun corfu--try-completion (str table pred pt &optional md)
+  "Complete STR given TABLE, predicate PRED, point PT and optional metadata MD."
+  (setq md (or md (completion-metadata (substring str 0 pt) table pred)))
+  (completion-try-completion str table pred pt md))
+
 (defsubst corfu--length-string< (x y)
   "Sorting predicate which compares X and Y first by length then by `string<'."
   (or (< (length x) (length y)) (and (= (length x) (length y)) (string< x y))))
@@ -612,28 +653,18 @@ FRAME is the existing frame."
           (eq t (compare-strings word 0 len it 0 len
                                  completion-ignore-case))))))
 
-;; bug#6581: `equal-including-properties' uses `eq' for properties until 29.1.
-;; Approximate by comparing `text-properties-at' position 0.
-(defalias 'corfu--equal-including-properties
-  (static-if (< emacs-major-version 29)
-      (lambda (x y)
-        (and (equal x y)
-             (equal (text-properties-at 0 x) (text-properties-at 0 y))))
-    #'equal-including-properties))
-
 (defun corfu--delete-dups (list)
   "Delete `equal-including-properties' consecutive duplicates from LIST."
   (let ((beg list))
     (while (cdr beg)
       (let ((end (cdr beg)))
         (while (equal (car beg) (car end)) (pop end))
-        ;; The deduplication is quadratic in the number of duplicates.  We can
-        ;; avoid the quadratic complexity with a hash table which takes
-        ;; properties into account (available since Emacs 28).
+        ;; The deduplication is quadratic in the number of duplicates.  We could
+        ;; avoid this via a hash table taking properties into account.
         (while (not (eq beg end))
           (let ((dup beg))
             (while (not (eq (cdr dup) end))
-              (if (corfu--equal-including-properties (car beg) (cadr dup))
+              (if (equal-including-properties (car beg) (cadr dup))
                   (setcdr dup (cddr dup))
                 (pop dup))))
           (pop beg)))))
@@ -957,11 +988,11 @@ See `completion-in-region' for the arguments BEG, END, TABLE, PRED."
   (when completion-in-region-mode (corfu-quit))
   (let* ((pt (max 0 (- (point) beg)))
          (str (buffer-substring-no-properties beg end))
-         (metadata (completion-metadata (substring str 0 pt) table pred))
-         (threshold (completion--cycle-threshold metadata))
+         (md (completion-metadata (substring str 0 pt) table pred))
+         (threshold (completion--cycle-threshold md))
          (completion-in-region-mode-predicate
           (or completion-in-region-mode-predicate #'always)))
-    (pcase (completion-try-completion str table pred pt metadata)
+    (pcase (corfu--try-completion str table pred pt md)
       ('nil (corfu--message "No match") nil)
       ('t (goto-char end)
           (corfu--message "Sole match")
@@ -981,17 +1012,16 @@ See `completion-in-region' for the arguments BEG, END, TABLE, PRED."
               (total (alist-get 'corfu--total state))
               (cands (alist-get 'corfu--candidates state)))
          (cond
-          ((<= total 1)
-           ;; If completion is finished and cannot be extended further and
-           ;; `corfu-on-exact-match' is not 'show, return 'finished.  Otherwise
-           ;; setup the popup.
-           (if (and (= total 1)
-                    (or (eq corfu-on-exact-match 'show)
-                        (consp (completion-try-completion
-                                newstr table pred newpt
-                                (completion-metadata newstr table pred)))))
+          ((= total 0)
+           (when (test-completion newstr table pred)
+             (corfu--exit-function newstr 'finished nil)))
+          ((= total 1)
+           ;; Setup popup if `corfu-on-exact-match' is `show' or if completion
+           ;; can continue.
+           (if (or (eq corfu-on-exact-match 'show)
+                   (consp (corfu--try-completion newstr table pred newpt)))
                (corfu--setup beg end table pred)
-             (corfu--exit-function newstr 'finished cands)))
+             (corfu--exit-function (car cands) 'finished nil)))
           ;; Too many candidates for cycling -> Setup popup.
           ((or (not threshold) (and (not (eq threshold t)) (< threshold total)))
            (corfu--setup beg end table pred))
@@ -1050,7 +1080,8 @@ See `completion-in-region' for the arguments BEG, END, TABLE, PRED."
                 (not buffer-read-only)
                 (corfu--match-symbol-p corfu-auto-commands this-command)
                 (corfu--popup-support-p))
-       (if (<= corfu-auto-delay 0)
+       (if (or (<= corfu-auto-delay 0)
+               (seq-contains-p corfu-auto-trigger last-command-event))
            (corfu--auto-complete-deferred)
          ;; Do not use `timer-set-idle-time' since this leads to
          ;; unpredictable pauses, in particular with `flyspell-mode'.
@@ -1082,36 +1113,37 @@ A scroll bar is displayed from LO to LO+BAR."
              (ml (min 16 (ceiling (* cw corfu-left-margin-width))))
              (mr (min 16 (ceiling (* cw corfu-right-margin-width))))
              (bw (min mr (ceiling (* cw corfu-bar-width))))
-             (fringe (display-graphic-p))
-             (marginl (and (not fringe) (propertize " " 'display `(space :width (,ml)))))
-             (sbar (if fringe
+             (graphic (display-graphic-p))
+             (marginl (and (not graphic) (propertize " " 'display `(space :width (,ml)))))
+             (sbar (if graphic
                        #(" " 0 1 (display (right-fringe corfu--bar corfu--bar)))
                      (concat
                       (propertize " " 'display `(space :align-to (- right (,bw))))
                       (propertize " " 'face 'corfu-bar 'display `(space :width (,bw))))))
-             (cbar (if fringe
+             (cbar (if graphic
                        #("  " 0 1 (display (left-fringe corfu--nil corfu-current))
                          1 2 (display (right-fringe corfu--bar corfu--cbar)))
                      sbar))
-             (cmargin (and fringe
+             (cmargin (and graphic
                            #("  " 0 1 (display (left-fringe corfu--nil corfu-current))
                              1 2 (display (right-fringe corfu--nil corfu-current)))))
              (pos (posn-x-y pos))
-             (width (+ (* width cw) (if fringe 0 (+ ml mr))))
+             (width (+ (* width cw) (if graphic 0 (+ ml mr))))
              ;; XXX HACK: Minimum popup height must be at least 1 line of the
              ;; parent frame (gh:minad/corfu#261).
              (height (max lh (* (length lines) ch)))
              (edge (window-inside-pixel-edges))
-             (border (alist-get 'internal-border-width corfu--frame-parameters))
+             (border (if graphic corfu-border-width 0))
              (x (max 0 (min (+ (car edge) (- (or (car pos) 0) ml (* cw off) border))
-                            (- (frame-pixel-width) width))))
+                            (- (frame-pixel-width) width
+                               (if graphic (+ ml mr (* 2 border)) 0)))))
              (yb (+ (cadr edge) (or (cdr pos) 0) lh
                     (static-if (< emacs-major-version 31) (window-tab-line-height) 0)))
              (y (if (> (+ yb (* corfu-count ch) lh lh) (frame-pixel-height))
                     (- yb height lh border border)
                   yb))
              (bmp (logxor (1- (ash 1 mr)) (1- (ash 1 bw)))))
-        (setq left-fringe-width (if fringe ml 0) right-fringe-width (if fringe mr 0))
+        (setq left-fringe-width (if graphic ml 0) right-fringe-width (if graphic mr 0))
         ;; Define an inverted corfu--bar face
         (unless (equal (and (facep 'corfu--bar) (face-attribute 'corfu--bar :foreground))
                        (face-attribute 'corfu-bar :background))
@@ -1217,14 +1249,14 @@ AUTO is non-nil when initializing auto completion."
            (equal (car corfu--candidates) str) (not (cdr corfu--candidates))
            (not (eq corfu-on-exact-match 'show))
            (or auto corfu-on-exact-match)
-           (not (consp (completion-try-completion str table pred pt corfu--metadata))))
+           (not (consp (corfu--try-completion str table pred pt))))
       ;; Quit directly when initializing auto completion.
       (if (or auto (eq corfu-on-exact-match 'quit))
           (corfu-quit)
         (corfu--done (car corfu--candidates) 'finished nil)))
      ;; 2) There exist candidates => Show candidates popup.
      (corfu--candidates
-      (let ((pos (posn-at-point (+ beg (length corfu--base)))))
+      (let ((pos (posn-at-point (min (point-max) (+ beg (length corfu--base))))))
         (corfu--preview-current beg end)
         (corfu--candidates-popup pos)))
      ;; 3) No candidates & `corfu-quit-no-match' & initialized => Confirmation popup.
@@ -1360,16 +1392,14 @@ If a candidate is selected, insert it.  Otherwise invoke
   (if (< corfu--index 0)
       (corfu-expand)
     ;; Continue completion with selected candidate.  Exit with status 'finished
-    ;; if input is a valid match and no further completion is
-    ;; possible. Additionally treat completion as finished if at the end of a
-    ;; boundary, even if other longer candidates would still match, since the
-    ;; user invoked `corfu-complete' with an explicitly selected candidate!
+    ;; if input is a valid match and no further completion is possible.
     (pcase-let ((`(,_beg ,_end ,table ,pred . ,_) completion-in-region--data)
                 (newstr (corfu--insert nil)))
       (and (test-completion newstr table pred)
-           (or (not (consp (completion-try-completion
-                            newstr table pred (length newstr)
-                            (completion-metadata newstr table pred))))
+           (or (not (consp (corfu--try-completion newstr table pred (length newstr))))
+               ;; Additionally finish completion if at the end of a boundary,
+               ;; even if other longer candidates match, since the user invoked
+               ;; `corfu-complete' with an explicitly selected candidate!
                (equal (completion-boundaries newstr table pred "") '(0 . 0)))
            (corfu--done newstr 'finished nil))
       t)))
@@ -1386,7 +1416,7 @@ input has been expanded."
     (pcase-let* ((`(,beg ,end ,table ,pred . ,_) completion-in-region--data)
                  (pt (max 0 (- (point) beg)))
                  (str (buffer-substring-no-properties beg end)))
-      (pcase (completion-try-completion str table pred pt corfu--metadata)
+      (pcase (corfu--try-completion str table pred pt)
         ('t
          (goto-char end)
          (corfu--done str 'finished corfu--candidates)
@@ -1397,9 +1427,7 @@ input has been expanded."
          ;; Exit with status 'finished if input is a valid match
          ;; and no further completion is possible.
          (and (test-completion newstr table pred)
-              (not (consp (completion-try-completion
-                           newstr table pred newpt
-                           (completion-metadata (substring newstr 0 newpt) table pred))))
+              (not (consp (corfu--try-completion newstr table pred newpt)))
               (corfu--done newstr 'finished corfu--candidates))
          t)))))
 
@@ -1433,17 +1461,6 @@ Quit if no candidate is selected."
     (remove-hook 'post-command-hook #'corfu--auto-post-command 'local)
     (kill-local-variable 'completion-in-region-function))))
 
-(defcustom global-corfu-modes t
-  "List of modes where Corfu should be enabled by `global-corfu-mode'.
-The variable can either be t, nil or a list of t, nil, mode
-symbols or elements of the form (not modes).  Examples:
-  - Enable everywhere, except in Org: ((not org-mode) t).
-  - Enable in programming modes except Python: ((not python-mode) prog-mode).
-  - Enable only in text modes: (text-mode)."
-  :type '(choice (const t) (repeat sexp))
-  :group 'corfu)
-
-;; TODO use `:predicate' on Emacs 29
 (defcustom global-corfu-minibuffer t
   "Corfu should be enabled in the minibuffer by `global-corfu-mode'.
 The variable can either be t, nil or a custom predicate function.  If
@@ -1456,21 +1473,14 @@ local `completion-at-point-functions'."
 (define-globalized-minor-mode global-corfu-mode
   corfu-mode corfu--on
   :group 'corfu
+  :predicate t
   (remove-hook 'minibuffer-setup-hook #'corfu--minibuffer-on)
   (when (and global-corfu-mode global-corfu-minibuffer)
     (add-hook 'minibuffer-setup-hook #'corfu--minibuffer-on 100)))
 
 (defun corfu--on ()
   "Enable `corfu-mode' in the current buffer respecting `global-corfu-modes'."
-  (when (and (not noninteractive) (not (eq (aref (buffer-name) 0) ?\s))
-             ;; TODO use `:predicate' on Emacs 29
-             (or (eq t global-corfu-modes)
-                 (eq t (cl-loop for p in global-corfu-modes thereis
-                                (pcase-exhaustive p
-                                  ('t t)
-                                  ('nil 0)
-                                  ((pred symbolp) (and (derived-mode-p p) t))
-                                  (`(not . ,m) (and (seq-some #'derived-mode-p m) 0)))))))
+  (unless (or noninteractive buffer-read-only (eq (aref (buffer-name) 0) ?\s))
     (corfu-mode)))
 
 (defun corfu--minibuffer-on ()
@@ -1508,5 +1518,9 @@ The ORIG function takes the FUN and WHICH arguments."
             :before-while #'corfu--eldoc-advice)
 (eldoc-add-command #'corfu-complete #'corfu-insert #'corfu-expand #'corfu-send)
 
+(with-eval-after-load 'corfu-terminal
+  (when (featurep 'tty-child-frames)
+    (display-warning 'corfu "`corfu-terminal' is not needed on Emacs 31")))
+
 (provide 'corfu)
 ;;; corfu.el ends here
